// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __typedefs1_hpp__
#define __typedefs1_hpp__

#include "typedefs_base.hpp"

namespace typedefs{

typedef item_t Item1;

typedef int my_int_t;

}

#endif//__typedefs1_hpp__
