// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __typedefs_hpp__
#define __typedefs_hpp__

namespace typedefs{

struct item_t{};

typedef item_t Item;

}

#endif//__typedefs_hpp__
