// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __const_volatile_arg_hpp__
#define __const_volatile_arg_hpp__

void pygccxml_bug(int const volatile* icv);

#endif//__const_volatile_arg_hpp__

