// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __core_ns_join_2_hpp__
#define __core_ns_join_2_hpp__

enum E21{ e21 };

namespace ns{

    enum E22{ e22 };

    namespace ns22{
        enum E23{ e23 };
    }

    namespace{
        enum E24{ e24 };
    }
}

#endif//__core_ns_join_2_hpp__
