// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

// A good test for different cpp standards. iostream is one of the first
// things to fail when the wrong flags are used.

#include <iostream>
