// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __typedefs2_hpp__
#define __typedefs2_hpp__

#include "typedefs_base.hpp"

namespace typedefs{

typedef item_t Item2;

}

#endif//__typedefs2_hpp__
