// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

int const c1 = 0;
const int c2 = 0;

volatile const int cv1 = 0;
const volatile int cv2 = 0;

const int * const cptr1 = 0;
