// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __complex_types_hpp__
#define __complex_types_hpp__

#ifndef WIN32
#include <complex.h>
#endif

#endif//__complex_types_hpp__


