// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __core_ns_join_3_hpp__
#define __core_ns_join_3_hpp__

enum E31{ e31 };

namespace ns{

    enum E32{ e32 };

    namespace ns32{
        enum E33{ e33 };
    }

    namespace{
        enum E34{ e34 };
    }
}

#endif//__core_ns_join_3_hpp__
