// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

class test
{
    public:
        // A constructor
        test(const float & t0){};

        void function(int arg1[1024], int arg2[512]) {};
};
