// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __declarations_for_filter_hpp__
#define __declarations_for_filter_hpp__

//those 2 includes wll bring a few declarations
#include <string>
#include <iostream>

namespace declarations{ namespace filter{

enum color{ red, green, blue };

} }

#endif//__declarations_for_filter_hpp__

