// Copyright 2014-2017 Insight Software Consortium.
// Copyright 2004-2009 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __include_all_hpp__
#define __include_all_hpp__

#include "core_cache.hpp"
#include "core_class_hierarchy.hpp"
#include "core_diamand_hierarchy_base.hpp"
#include "core_diamand_hierarchy_derived1.hpp"
#include "core_diamand_hierarchy_derived2.hpp"
#include "core_diamand_hierarchy_final_derived.hpp"
#include "core_membership.hpp"
#include "core_ns_join_1.hpp"
#include "core_ns_join_2.hpp"
#include "core_ns_join_3.hpp"
#include "core_overloads_1.hpp"
#include "core_overloads_2.hpp"
#include "core_types.hpp"
#include "declarations_calldef.hpp"
#include "declarations_comparison.hpp"
#include "declarations_enums.hpp"
#include "declarations_variables.hpp"

#endif//__include_all_hpp__


